# frozen_string_literal: true

module Stormbreaker
  class AxeViolationManager
    attr_accessor :total_violations, :test_violations

    def initialize
      self.total_violations = []
      self.test_violations = []
    end

    def add_failure(called_by: Set.new, violation: nil, node: nil)
      axe_violation = get_violation(called_by: called_by, violation: violation, node: node)
      add_failure_to_test(axe_violation)
      add_failure_to_total(axe_violation)
    end

    def get_violation(called_by: Set.new, violation: nil, node: nil)
      raise 'No violation found' unless violation && node

      severity = violation.impact
      element = node.html
      rule = violation.description
      complete_summary = node.failure_messages.flatten.join("\n")

      AxeViolation.new(called_by: called_by, severity: severity, element: element, violation: rule,
                       complete_summary: complete_summary)
    end

    def add_failure_to_test(axe_violation)
      @test_violations << axe_violation
    end

    def add_failure_to_total(axe_violation)
      match = @total_violations.select { |v| (v.violation == axe_violation.violation && v.element == axe_violation.element) }
      raise "Multiple total_violations Found for #{v.description}" if match.count > 1

      if match.any?
        match.first.called_by = match.first.called_by.merge(axe_violation.called_by)
      else
        @total_violations << axe_violation
      end
    end

    def clear_failures_from_test
      @test_violations = []
    end

    def clear_failures_from_total
      @total_violations = []
    end

    def test_passed?
      @test_violations.empty?
    end

    def total_passed?
      @total_violations.empty?
    end

    def sort_test_violations
      sorted = []
      Stormbreaker.configuration.enabled_severity_categories.each do |severity|
        sorted += @test_violations.select { |v| v.severity == severity }.sort_by(&:violation)
      end
      sorted
    end

    def list_test_results
      summary = ''
      prev_violation = ''
      sort_test_violations.each do |violation|
        summary += <<~INDENT
          #{prev_violation != violation.violation ? "\n#{violation.violation} - Severity: #{violation.severity}\n" : ''}
            #{violation.complete_summary.gsub("\n", "\n  ")}

            Violations Found In:
              #{violation.called_by.to_a.join("\n")}
        INDENT
        prev_violation = violation.violation
      end
      summary
    end

    def summarize_total_results_by_severity
      summary = ''
      error_categories = Stormbreaker.configuration.enabled_severity_categories
      unique_violations = 0

      error_categories.each do |severity|
        violation_map = {}
        @total_violations.select { |v| v.severity == severity }.each do |error|
          if violation_map[error.violation]
            violation_map[error.violation] += error.called_by.count
          else
            unique_violations += 1
            violation_map[error.violation] = error.called_by.count
          end
        end

        next if violation_map.empty?

        summary += "\n\n  Level: #{severity}"
        violation_map.each do |error, count|
          summary += "\n   - #{error} : #{count} instance(s)"
        end
      end
      "\nFound #{unique_violations} unique violation(s)" + summary
    end
  end
end
